st7/
;------------------------------------------------------------------------------
; http://st7-mcu.dxp.pl/
; Subject : Interfacing with M24C64 I2C EEPROM memory
; File : M24C64.asm
; (C) Radoslaw Kwiecien, radek@dxp.pl
;------------------------------------------------------------------------------
	#include "I2C.inc"
	
M24SLAVEADDR EQU 	%10100000
M24READADDR	 EQU	{M24SLAVEADDR + 1}
M24WRITEADDR EQU	{M24SLAVEADDR + 0}

;------------------------------------------------------------------------------
; Data RAM space
;------------------------------------------------------------------------------
	BYTES
	segment 'ram0'
.m24DstAddr	DS.w 	1 ; source address
.m24SrcAddr	DS.W 	1	; destination address
m24Count		DS.B 	1 ; byte count
	
;------------------------------------------------------------------------------
; Program flash space
;------------------------------------------------------------------------------
	WORDS
	segment 'rom'
;------------------------------------------------------------------------------
; Parameters
; 	m24DstAddr : destination address
;   A : data to write
;------------------------------------------------------------------------------
.M24C64_WriteByte
	PUSH	A												; store data on stack
	CALL	I2C_Start								; START
	LD		A, #M24WRITEADDR				;
	CALL	I2C_Write								; address of M24C64
	LD		A, {m24DstAddr + 0}			;
	CALL	I2C_Write								;	MSB of memory address
	LD		A, {m24DstAddr + 1} 		;
	CALL	I2C_Write								;	LSB	of memory address
	POP		A												; restore data from stack
	CALL	I2C_Write								; data write
	CALL  I2C_Stop								; STOP
	RET

;------------------------------------------------------------------------------
; Parameters
;   m24SrcAddr : source address (buffer in RAM)
; 	m24DstAddr : destination address (in EEPROM)
; 	A : bufor size (max. 32 bytes)
;------------------------------------------------------------------------------
.M24C64_WriteBuffer
	LD		m24Count, A							; number of bytes to write
	CALL	I2C_Start								; START
	LD		A, #M24WRITEADDR				;
	CALL	I2C_Write								; address of M24C64
	LD		A, {m24DstAddr + 0}			;
	CALL	I2C_Write								;	MSB destination address
	LD		A, {m24DstAddr + 1} 		;
	CALL	I2C_Write								;	LSB	destination address
	CLR		X												;
M24C64_WriteBufferLoop					;
	LD		A,([m24SrcAddr.w],X)		;	read from buffer
	CALL	I2C_Write								; write to EEPROM
	INC		X												;	X++
	CP		X, m24Count							; if last byte?
	JRNE	M24C64_WriteBufferLoop	;	if not, jump to loop
	CALL	I2C_Stop								; STOP
	RET

;------------------------------------------------------------------------------
; Parameters
;   m24SrcAddr : source address (in EEPROM)
; Return value
;	  A : read data
;------------------------------------------------------------------------------
.M24C64_ReadByte
	CALL	I2C_Start								; START
	LD		A, #M24WRITEADDR				;
	CALL	I2C_Write								; slave address
	LD		A, {m24SrcAddr + 0}			;
	CALL	I2C_Write								; MSB source address
	LD		A, {m24SrcAddr + 1}			;
	CALL	I2C_Write								; LSB source address
	CALL	I2C_Start								; repeated START
	LD		A, #M24READADDR					;
	CALL	I2C_Write								; slave address + Read
	SCF														; NOACK
	CALL	I2C_Read								; data read
	CALL  I2C_Stop								; STOP
	RET
	
;------------------------------------------------------------------------------
; Parameters
; 	m24SrcAddr : source address
; 	m24DstAddr : destination address
; 	A : buffer size (min. 2)
;------------------------------------------------------------------------------
.M24C64_ReadBuffer
	DEC		A												;
	LD		m24Count, A							; buffer size - 1
	CALL	I2C_Start								; START
	LD		A, #M24WRITEADDR				;
	CALL	I2C_Write								; slave address
	LD		A, {m24SrcAddr + 0}			;
	CALL	I2C_Write								; MSB of source adress
	LD		A, {m24SrcAddr + 1}			;
	CALL	I2C_Write								; LSB of source address
	CALL	I2C_Start								; START
	LD		A, #M24READADDR					;
	CALL	I2C_Write								; slave address + R
	CLR		X												
M24C64_ReadBufferLoop
	RCF														; ACK
	CALL	I2C_Read								; read byte from EEPROM
	LD		([m24DstAddr.w],X), A		; write to bufer
	INC		X												; X++
	CP		X, m24Count							; is one before last byte?
	JRNE	M24C64_ReadBufferLoop		; if not jump to loop
	SCF														; NOACK
	CALL	I2C_Read								; read last byte from EEPROM
	LD		([m24DstAddr.w],X), A		; write to buffer
	CALL	I2C_Stop								; STOP
	RET

;------------------------------------------------------------------------------
; Procedura oczekujca na bit ACK (zakoczenie operacji programowania)
;------------------------------------------------------------------------------
.M24C64_WaitForACK
	CALL	I2C_Start								; START
	LD		A, #M24WRITEADDR				;
	CALL	I2C_Write								; Adres ukadu M24C64
	JRC		M24C64_WaitForACK				;	Jeli odebrano NOACK to skok na pocztek
	CALL	I2C_Stop
	RET

	END
;------------------------------------------------------------------------------
; Koniec pliku M24C64.asm
;------------------------------------------------------------------------------
